#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
#############################################################
#                                                           #
#      Copyright @ 2019 -  Osirish Network.                 #
#      All rights reserved.                                 #
#                                                           #
#      osirish                                          #
#                                                           #
#                                                           #
#############################################################
#
#
#  @Author: https://bit.ly/osirish
#


import os
import multiprocessing
import argparse
import time
import sys
import random
import re
import json
import requests
import io

# from colorama import coloramaInit, Fore, Style
from OsirishConfig import cwd, plugins_dir, auth_api, config_file, configuration
from OsirishAPI import OsirishAPI
from OsirishMachine import OsirishMachine
from OsirishUtils import createDir, getFile, \
    deleteFile, putFile, getInput, getMenu, creation_date, \
    id_generator, myexcepthook
from OsirishRegex import RGATES_URL
from colorama import coloramaInit, Fore, Style
coloramaInit(autoreset=True)

user = {}
osirishengine = []
__plugins__ = [{"id": "__install__", "name": "Install plugin"}]
temp = {}

def logoutMenu():
    global user
    deleteFile(auth_api)
    user = {}
    return authMenu()

def homeMenu():
    global user, __plugins__
    if user['Balance'] > 0:
        print(Fore.GREEN + "You have {}$ USD in your account!".format(user['Balance']))

    if user['Credits'] > 0:
        print(Fore.GREEN + "You have {} Credits in your account!".format(user['Credits']))

    a = getMenu("Welcome " + user['Username'] + "! What do you want to do today?", [
        {"id": 'checker', "name": "Checker"},
        {"id": 'tools', "name": "Tool & Plugin"},
        {"id": 'contact', "name": "Contact"},
        {"id": 'logout', "name": "Logout"}
    ])

    if a == 'checker': checkerMenu()
    elif a == 'contact': 
        try:
            print(Fore.YELLOW + "Contact me at osirish.net@gmail.com")
            time.sleep(2)

        except(KeyboardInterrupt, SystemExit):
            sys.exit(0)
            
    elif a == "tools":
        try:
            b = getMenu("Your tool & plugins list (" + str(len(__plugins__) - 1) + "):", __plugins__)
            if b == "__install__":
                return OsirishPluginsInstaller()
            sys.path.append(plugins_dir)
            getattr(__import__(b, fromlist=["onLoad"]), 'onLoad')()
            
        except(KeyboardInterrupt, SystemExit):
            sys.exit(0)

        except:
            pass

    elif a == 'logout':
        logoutMenu()

    return homeMenu()

def checkerMenu():
    global options, user, osirishengine

    # Input CheckerID
    CheckerID = (options.checkerid is not None and options.checkerid or getInput("Checker ID: ", r"(^(?!:\/\/)([a-zA-Z0-9-_]+\.)*[a-zA-Z0-9][a-zA-Z0-9-_]+\.[a-zA-Z]{2,11}?$)|(^[a-zA-Z0-9]+$)")).lower()
    sys.stdout.write("Getting configuration for " + CheckerID + " ... ")
    sys.stdout.flush()

    # Get Checker Configuration
    a = OsirishAPI().get("getChecker", {"CheckerID": CheckerID}).parse()

    # Prepare Result DIR
    result_dir = os.path.realpath(cwd + "/results/" + CheckerID)
    if createDir(result_dir) == False: print("Unable to create result directory")

    # Verify checker
    if "name" in a and "author" in a:
        # Configuration
        delim = "|"
        jobs = []
        resource = []

        # print(a['match'])
        print("Done")
        print(Fore.GREEN + "Current Directory: " + os.getcwd())
        sk = options.resource is not None and options.resource or getInput("Input resource file: ")
        if os.path.isfile(sk + ".txt") and not os.path.isfile(sk):
            sk += ".txt"

        # Get Timeout
        timeout = options.timeout is not None and options.timeout or configuration['checker']['timeout']
        
        # Machine Configuration
        machine = options.machine == 0 and int(getInput("Total machine to be spawned: ", r"[0-9]+")) or options.machine
        jobs = []

        # Start Checking
        print("Starting Checking ... \n")
        with open(sk, 'r', encoding="utf-8", errors='replace') as f:
            for dline in f:
                Scan = False
                line = dline.replace("\n", "").replace("\t", "").replace(" ", "")
                if a['ResourceFormat'] == "all":
                    Scan = True

                else:
                    for dk in re.compile(a['match']).finditer(line):
                        # Get Delimiter 
                        try:
                            delim = dk.group('delimiter')

                        except:
                            pass

                        # Get Resource
                        try:
                            Scan = True
                            line = dk.group('resource')

                        except Exception as exy:
                            print("Error while parsing resource! ({0})".format(exy))
                
                # Get Resource and Start Check
                if Scan:
                    try:
                        # Waiting for open slot
                        CheckingJobs = True
                        while CheckingJobs:
                            # Check for alives
                            for n, job in enumerate(jobs):
                                if not job.is_alive():
                                    del jobs[n]
                            
                            # If slot available
                            if len(jobs) < machine:
                                CheckingJobs = False
                                process = multiprocessing.Process(target=OsirishMachine(
                                        id=len(jobs) + 1,
                                        resource=[line],
                                        mid=id_generator(10),
                                        engine=osirishengine,
                                        token=user['Token'],
                                        cid=CheckerID,
                                        options=options,
                                        result_dir=result_dir,
                                        user=user,
                                        delim=delim,
                                        timeout=timeout,
                                        configuration=configuration['checker']
                                    ).start,
                                args=())

                                # Append to jobs
                                jobs.append(process)

                                # Configuration
                                process.daemon = True
                                sys.excepthook = myexcepthook

                                # Start
                                process.start()
                        
                    except(KeyboardInterrupt, SystemExit):
                        sys.exit(0)
                        
                    except Exception as exy:
                        print("Error while sending resource to machine! ({0})".format(exy))

            print("Checking Done!")
    else:
        options.checkerid = None
        print("NOT FOUND!")
        return checkerMenu()

def OsirishPluginsInstaller():
    global options
    AppName = options.resource is None and getInput("Plugin ID: ") or options.resource
    sys.stdout.write("Installing {} ... ".format(AppName))
    sys.stdout.flush()
    try:
        for x in configuration['plugins']['repository']:
            Script = OsirishAPI(url=x).get(AppName + ".py")
            if Script.code() == 200:
                if os.path.isfile(plugins_dir + "/" + AppName + ".py"):
                    print("EXIST!")

                else:
                    putFile(plugins_dir + "/" + AppName + ".py", Script.text())
                    print("Done")
                    try:
                        sys.path.append(plugins_dir)
                        __plugins__.append({'id': AppName, 'name': getattr(__import__(AppName, fromlist=["__name__"]), '__name__')})

                    except:
                        print("Unable to load plugin " + AppName)
                return homeMenu()
        print("NOT FOUND!")
        return homeMenu()
        
    except(KeyboardInterrupt, SystemExit):
        sys.exit(0)

    except:
        pass

def logged():
    global osirishengine, user, auth_api
    a = getFile(auth_api)
    if a != "":
        sys.stdout.write("Logging in with cache ... ")
        sys.stdout.flush()
        da = OsirishAPI().get("getToken", {"api": a}).parse()
        if "user" in da:
            user = da['user']
            osirishengine = da['engine']
            if "Username" in user:
                print("Done")
                if "news" in da:
                    if len(da['news']) > 0:
                        for x in da['news']:
                            print(x)
                return True

            else:
                print("FAILED!")
                putFile(auth_api, '')
                return False

        else:
            print("FAILED!")
            putFile(auth_api, '')
            return False
    return False

def authMenu(a = None):
    global osirishengine, user
    if a is None:
        if logged():
            return True
        return authMenu(getMenu("Please authenticate yourself.", [{"id": "login", "name": "Login"}]))
    
    if "login" in a.lower():
        print("To get API Key open (https://www.osirish.club/account/?page=api-configuration) and click Generate API")
        b = getInput("API Key: ")
        d = OsirishAPI().get("getToken", {"api": b}).parse()
        if "user" in d:
            putFile(auth_api, b)
            return authMenu(None)

        else:
            if "msg" in d:
                print(d['msg'])
                if "invalid" in d['msg'].lower() or "username doesn" in d['msg'].lower():
                    return authMenu('login')

            else:
                print("Unable to connect Osirish Network Server")
                
    else:
        print("Not Found")
    
    return authMenu(None)

def mainEntry():
    global options, __plugins__
    parser = argparse.ArgumentParser(
        description="Osirish Network - Underground internet tool, useful for cracking, carding. Making your work easier and faster.")
    parser.add_argument('-d', '--do',
        type=str,
        help="What do you want to do today?", default="", choices=["check", "plugins", "add_repo"])
    parser.add_argument('-cid', '--checkerid', type=str, help="Checker ID to check the resource", default=None)
    parser.add_argument('-t', '--timeout', type=int, help="Checking timeout", default=0)
    parser.add_argument('-r', '--resource', type=str, help="Resource file or name", default=None)
    parser.add_argument('--delim', type=str, help="Delimiter", default=None)
    parser.add_argument('-m', '--machine', type=int, help="How much machine to spawn", default=0)
    parser.add_argument('--max-retry', type=int, help="Max retry for failed checking", default=30)
    parser.add_argument('--hash', type=str, help="Password Hash", default=None)
    options = parser.parse_args()

    for name in os.listdir(plugins_dir):
        if name.endswith(".py"):
            #strip the extension
            module = name[:-3]
            
            # set the module name in the current global name space:
            try:
                sys.path.append(plugins_dir)
                __plugins__.append({'id': module, 'name': getattr(__import__(module, fromlist=["__name__"]), '__name__')})

            except(KeyboardInterrupt, SystemExit):
                sys.exit(0)
                
            except:
                print("Unable to load plugin " + module)

    if authMenu():
        if options.do.lower() == "check":
            if options.checkerid is not None:
                checkerMenu()

            else:
                homeMenu()

        elif options.do.lower() == "plugins":
            OsirishPluginsInstaller()

        elif options.do.lower() == "add_repo":
            ResUrl = options.resource == "" and getInput("URL: ") or options.resource
            b = True
            if ResUrl[:-1] != "/": ResUrl += "/"
            for x in configuration['plugins']['repository']:
                if x == ResUrl:
                    b = False
                    
            if b == True:
                configuration['plugins']['repository'].append(ResUrl)
                putFile(config_file, json.dumps(configuration, sort_keys=True, indent=4))
            print("Success")

        else:
            homeMenu()

    else:
        print("Unable to view your information")

if __name__ == "__main__":        
    mainEntry()